import pickle

def DumpRawDataObj(rawDataObj,file):
	"""
	Dumping the rawDataObj into .pickle file

	:param rawDataObj: ds_InputRawData
	:type rawDataObj: object
	:returns: file
	"""
	with open(file, 'wb') as f:
		pickle.dump(rawDataObj, f, pickle.HIGHEST_PROTOCOL)
		print('.....Successfully dump raw data object at {}\n'.format(file))

def DumpProtClustObj(protClustObj,file):
	"""
	Dumping the protClustObj into .pickle file

	:param protClustObj: ds_InputRawData
	:type protClustObj: object
	:returns: file
	"""
	with open(file, 'wb') as f:
		pickle.dump(protClustObj, f, pickle.HIGHEST_PROTOCOL)
		print('.....Successfully dump protein cluster object at {}\n'.format(file))

def LoadRawDataObj(file):
	"""
	Loading a .pickle file to fast create a rawDataObj

	:param file:
	:type file: .pickle
	:returns: rawDataObj
	"""
	with open(file, 'rb') as f:
		rawDataObj = pickle.load(f)
		print('.....Successfully read raw data object from {}\n'.format(file))
	return rawDataObj

def LoadProtClustObj(file):
	"""
	Loading a .pickle file to fast create a protClustObj

	:param file:
	:type file: .pickle
	:returns: protClustObj
	"""
	with open(file, 'rb') as f:
		protClustObj = pickle.load(f)
		print('.....Successfully read protein cluster object from {}\n'.format(file))
	return protClustObj


from .CommonFunction import *
import matplotlib.pyplot as plt


def DrawDivergeBarForCombineFig(protClustObj, currentFeatureSer, axes):
    """
    draw a diverge bar of the entropy/backgroundEntropy of each cluster

    :param currentFeatureSer: the Series includes each sample's feature value. ['white','black','white']
    :param axes: the postion of the bar plot in the combineFig
    :return: axes, the postion of the bar plot in the combineFig
    """
    center = 1 #the center of the score(since the average distribute of every feature value's entropy/backgroundEntropy is 1)
    currentFeatureName = currentFeatureSer.name
    sampleClustSer = protClustObj.clustResultObj.sampleClustSer
    scoringLi = _GetEntropyFromProtClust(protClustObj, sampleClustSer, currentFeatureName)
    clustXaxisLi = GetClustXaxisLi(sampleClustSer)
    ##### maybe run different scoring results beside entropy,so here need to extend
    # ScoringLi =__divideBackground(clustNumSer, currentFeautreSer)

    maxValue, minValue = _DetermineMaxMinValue(scoringLi)
    yAxisRange = _DetermineYaxisRange(maxValue, minValue, center)
    entropyLineColor = _DetermineLineColor(scoringLi, center)

    # Draw plot
    # plt.figure(figsize=(14,10), dpi= 80)
    axes.vlines(x=GetXlabelPositionLi(sampleClustSer), ymin=center, ymax=scoringLi, color=entropyLineColor, alpha=1,
                linewidth=15)

    axes.set_ylim(center - yAxisRange, center + yAxisRange)
    axes.set_ylabel('entropy')
    axes.set_xlabel('cluster')
    axes.set_xticks(GetXlabelPositionLi(sampleClustSer))
    axes.set_xticklabels(clustXaxisLi)
    axes.grid(linestyle='--', alpha=0.5)
    axes.axhline(y=center, color='black', linestyle='solid')
    axes.set_title('Diverge Bar')
    return axes


def DrawDivergeBar(protClustObj, currentFeatureSer, figSizeLi=[5, 4], dpi=100, path='divergebar.png'):
    """
    draw a diverge bar of the entropy/backgroundEntropy of each cluster

    :param currentFeatureSer: the Series includes each sample's feature value. ['white','black','white']
    :param figSizeLi: the width and high of the figure
    :param dpi: Dots Per Inch in the figure
    :return: fig, the diverge bar subplot fig
    """
    center = 1  #the center of the score(since the average distribute of every feature value's entropy/backgroundEntropy is 1)
    currentFeatureName = currentFeatureSer.name
    clustNumSer = protClustObj.clustResultObj.sampleClustSer
    clustXaxisLi = GetClustXaxisLi(clustNumSer)
    scoringLi = _GetEntropyFromProtClust(protClustObj, clustNumSer, currentFeatureName)
    fig, axes = plt.subplots(1, 1, figsize=(figSizeLi[0], figSizeLi[1]), dpi=dpi)
    ##### maybe run different scoring results beside entropy,so here need to extend
    # ScoringLi =__divideBackground(clustNumSer, currentFeautreSer)

    maxValue, minValue = _DetermineMaxMinValue(scoringLi)
    yAxisRange = _DetermineYaxisRange(maxValue, minValue, center)
    entropyLineColor = _DetermineLineColor(scoringLi, center)

    # Draw plot

    axes.vlines(x=GetXlabelPositionLi(clustNumSer), ymin=center, ymax=scoringLi, color=entropyLineColor, alpha=1,
                linewidth=15)

    axes.set_ylim(center - yAxisRange, center + yAxisRange)
    axes.set_ylabel('entropy')
    axes.set_xlabel('cluster')
    axes.set_xticks(GetXlabelPositionLi(clustNumSer))
    axes.set_xticklabels(clustXaxisLi)
    axes.grid(linestyle='--', alpha=0.5)
    axes.axhline(y=center, color='black', linestyle='solid')
    axes.set_title('Diverge Bar')

    fig.savefig(path, bbox_inches="tight")

    return fig


def _DetermineLineColor(scoringLi, center):
    """
    determine the color of the line in the diverge bar, if > center is green, otherwise is red

    :param scoringLi: a scoring list contain score each cluster
    :param center: the y-axis center
    :return: color list (green of red)
    """
    colorLi = ['red' for i in range(len(scoringLi))]
    for i in range(len(scoringLi)):
        if scoringLi[i] > center:
            colorLi[i] = 'green'

    return colorLi


def _DetermineYaxisRange(maxValue, minValue, center=1):
    """
    use max value and min value to determine the range of the y-axis of the diverge plot

    :param maxValue: the max value  of the Scoring list
    :param minValue: the minimum value
    :param center: the y-axis center
    :return: float, the range for y-axis, will later set into (center+range, center-range) in DrawDivergeBar function
    """
    maxRange = abs(maxValue - center)
    minRange = abs(minValue - center)

    if maxRange >= minRange:
        return maxRange + 0.1
    elif maxRange < minRange:
        return minRange + 0.1


def _DetermineMaxMinValue(ScoringLi):
    """
    determine the max value and minimum value of the Scoring list

    :param ScoringLi: a scoring list contain score each cluster
    :return: max value, min value
    """
    maxValue = ScoringLi[0]
    minValue = ScoringLi[0]
    for i in range(len(ScoringLi)):
        if ScoringLi[i] < minValue:
            minValue = ScoringLi[i]
        elif ScoringLi[i] > maxValue:
            maxValue = ScoringLi[i]

    return maxValue, minValue


def _GetEntropyFromProtClust(protClustObj, clustNumSer, currentFeatureName):
    """
    get entropy series from protClustObj in order to draw diverge bar

    :param protClustObj: ds_protClust object
    :param clustNum:int, the number of clusters
    :param currentFeatureName: string ,the feature name
    :return: entropy list
    """
    clustNum = len(set(clustNumSer))
    scoreLi = [0.0 for x in range(clustNum)]
    scoreCount = 0
    for currentClustCount in protClustObj.clustScoringObj.clustFeatureEntropySer.index.values:
        scoreLi[scoreCount] = protClustObj.clustScoringObj.clustFeatureEntropySer[currentClustCount][
                                  currentFeatureName] / protClustObj.clustScoringObj.featureEntropyDi[
                                  currentFeatureName]
        scoreCount += 1

    return scoreLi



# -*- coding: utf-8 -*-
"""
Created on Tue Oct  8 14:19:08 2019

@author: tea9262
"""
from scipy.cluster.hierarchy import dendrogram, linkage, cophenet, fcluster
from scipy.spatial.distance import pdist
from matplotlib import pyplot as plt

import numpy as np
import pandas as pd
from ProteomeClustering import DataAdapter
from ProteomeClustering import ClusteringMethod
from ProteomeClustering import Characterization
from ProteomeClustering import Statistics
from ProteomeClustering import Visualization
from ProteomeClustering import CaseStudy
from ProteomeClustering import Scoring
from ProteomeClustering.DataStructure import ds_InputRawData
from ProteomeClustering.DataStructure import ds_ProtClust
from scipy import stats
from sklearn.metrics import adjusted_rand_score
from sklearn.metrics.cluster import v_measure_score

import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from sklearn import metrics
from pandas.core.frame import DataFrame
import random


### DataAdapter###

def PreprocessRawData_RemoveNa():
    """
    ReadFilesPath will check the number of patients is equal in protein ratio and metadata, remove continuous columns of
    metadataDf and set missing value into NA, if you want to remove missing value, you can use RemoveProtWithNa function
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)

    print(protDataObj.GetProtRatioDf())
    print(protDataObj.GetMetadataDf())


def PreprocessRawData_Imputation():
    """
    Besides remove missing value, you can also choose to impute missing value, there are three functions ImputeMin,
    ImputeAverage and ImputeKnn.
    """

    #####   InputRawData test
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    #protDataObj = DataAdapter.ImputeMin(protDataObj, percentage=0.1)
    #protDataObj = DataAdapter.ImputeAverage(protDataObj, percentage=0.1)
    protDataObj = DataAdapter.ImputeKnn(protDataObj, n_neighbors=5, weightStr='uniform', percentage=0.1)

    print(protDataObj.GetProtRatioDf())
    print(protDataObj.GetMetadataDf())


def PreprocessRawData_Log():
    """
    Before we run clustering, you can use RatioToLogRatio() to change your protein ratio data into log ratio or use
    LogRatioToRatio() to change log ratio data into ratio data.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.LogRatioToRatio(protDataObj)
    protDataObj = DataAdapter.RatioToLogRatio(protDataObj)

    print(protDataObj.GetProtRatioDf())
    print(protDataObj.GetMetadataDf())


def PreprocessRawData_Normalize():
    """
    Before we run clustering, you can use ZscoreNmlz() to z-score your protein ratio data.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.ZscoreNmlz(protDataObj)

    print(protDataObj.GetProtRatioDf())
    print(protDataObj.GetMetadataDf())


def PreprocessRawData_Filter():
    """
        Before we run clustering, you can useFilterByRank(),  FilterByCutoff() or  FilterByTopPercentage() to
        select features.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, 0.5, methodStr='SD')
    #protDataObj = DataAdapter.FilterByTopPercentage(protDataObj, 0.1, methodStr='SD')
    #protDataObj = DataAdapter.FilterByRank(protDataObj, 1000, methodStr='SD')

    print(protDataObj.GetProtRatioDf())
    print(protDataObj.GetMetadataDf())



### Clustering###

def Clustering_Hierarchical():
    """
    Build a new object protClustObj to save the clustering result and then run hierarchical clustering
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()

    protClustObj.clustParamObj.SetParam(method="hierarchicalClust", linkage="average",
                                            distance="euclidean", clustNum=5)
    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    print(protClustObj.clustResultObj.sampleClustSer)


def Clustering_KMeans():
    """
    Build a new object protClustObj to save the clustering result and then run K-means clustering.
    'kmeansClust_sklearn' can only use euclidean distance,  'kmeansClust_nltk' can use euclidean, manhattan, and cosine.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    seed = random.randint(1, 10000)
    protClustObj.clustParamObj.SetParam(method="kmeansClust_sklearn", seed=seed, clustNum=5)
    #protClustObj.clustParamObj.SetParam(method="kmeansClust_nltk", kmeansDisatance="manhattan", clustNum=5)
    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    print(protClustObj.clustResultObj.sampleClustSer)


def Clustering_Gaussian():
    """
    Build a new object protClustObj to save the clustering result and then run Gaussian Model clustering.
    covType(covariance_type) can choose 'full', 'tied', 'diag' or 'spherical'
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()

    protClustObj.clustParamObj.SetParam(method="gaussianEMClust", clustNum=5, covType='full')
    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    print(protClustObj.clustResultObj.sampleClustSer)


def Clsutering_ConsensusHierarchical():
    """
    Build a new object protClustObj to save the clustering result and then run Consensus Hierarchical clustering.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='hierarchicalClust',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean',
                                        ccLinkage='ward')

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    print(protClustObj.clustResultObj.sampleClustSer)


def Clsutering_ConsensusKmeans():
    """
    Build a new object protClustObj to save the clustering result and then run Consensus Kmeans clustering.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(ccMethod='kmeansClust_nltk', method="consensusClust",
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean')

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)
    print(type(protClustObj))
    print(protClustObj.clustResultObj.sampleClustSer)


### Scoring ###

def Scoring_CalGiniScore():
    """
    Calculate the gini score of a consensus kmeans clutsering result.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                    minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean', )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Scoring.CalcGiniScore(protDataObj, protClustObj)
    print("giniScore: " + str(protClustObj.clustScoringObj.giniScore))


def Scoring_CalEveryScore():
    """
    Use CalEveryScore to calculate every scoring function we have, some are only related to clustering result like gini
    score and sihouette score, others are related to clustering result and meta data.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean')

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Scoring.CalcEveryScore(protDataObj, protClustObj)

    # print scoring results
    print("partitionIndex: "+str(float(protClustObj.clustScoringObj.partitionIndex)))
    print("purity: " + str(protClustObj.clustScoringObj.purity))
    print("separation index: " + str(protClustObj.clustScoringObj.separationIndex))
    print("giniScore: " + str(protClustObj.clustScoringObj.giniScore))
    print("silhScore: " + str(protClustObj.clustScoringObj.silhScore))
    print("clustSilhScoreSer: " + str(protClustObj.clustScoringObj.clustSilhScoreSer))
    print("sampleSilhScoreSer: " + str(protClustObj.clustScoringObj.sampleSilhScoreSer))
    print("clustFeatureEntropySer: "+str(protClustObj.clustScoringObj.clustFeatureEntropySer))
    print("featureAriDi: " + str(protClustObj.clustScoringObj.featureAriDi))
    print("featureAmiDi: " + str(protClustObj.clustScoringObj.featureAmiDi))
    print("featureEntropyDi: " + str(protClustObj.clustScoringObj.featureEntropyDi))
    print("featureMaxFscoreDi: " + str(protClustObj.clustScoringObj.featureMaxFscoreDi))
    print("featureVscoreDi: " + str(protClustObj.clustScoringObj.featureVscoreDi))
    print("clustFeatureValueRatioSer: "+str(protClustObj.clustScoringObj.clustFeatureValueRatioSer))
    print("featurePairedFscoreDi: " + str(protClustObj.clustScoringObj.featurePairedFscoreDi))







### Visualization ###

def Visualization_CombineFig():
    """
    choose two interested features in metadata to see the connection between features and clustering result by the
    diverge bar, stacked bar and scatter plot together.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean', )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    ### draw combine figure, choose two metadata feature to compare, needs to count entropy first
    protClustObj = Scoring.CalcClustFeatureEntropy(protDataObj, protClustObj)
    protClustObj = Scoring.CalcFeatureEntropy(protDataObj, protClustObj)
    currentFeatureSer = protDataObj.GetMetadataDf()['Stage']
    compareFeatureSer = protDataObj.GetMetadataDf()['pathology_N_stage']
    Visualization.DrawCombinedFig(protClustObj, currentFeatureSer, compareFeatureSer,  figTitleStr='Combined Figure',
                    figSizeLi=[15, 4], dpi=96, wSpace=0.7, pathStr='combinFig.png',
                    colorsStr='Set1', orderLi=['stacked', 'scatter', 'diverge'])


def Visualization_Featureplot():
    """
    You can also draw diverge bar, stacked bar and scatter plot separately.
    Diverge bar is to see the difference of entropy between each cluster
    Stacked bar is to see the number the feature values between each cluster
    Scatter plot is to see two features' distribution between each cluster
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean')

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    ### draw feature plot, choose two metadata feature to compare, needs to count entropy first
    protClustObj = Scoring.CalcClustFeatureEntropy(protDataObj, protClustObj)
    protClustObj = Scoring.CalcFeatureEntropy(protDataObj, protClustObj)
    currentFeatureSer = protDataObj.GetMetadataDf()['Stage']
    compareFeatureSer = protDataObj.GetMetadataDf()['pathology_N_stage']

    ### draw scatterplot, stackedbar, divergebar , respectively
    Visualization.DrawDivergeBar(protClustObj, currentFeatureSer, figSizeLi=[5, 4], dpi=96, path='divergebar.png')
    Visualization.DrawStackedBar(protClustObj, currentFeatureSer, color="Set1", edgeColor='white', width=0.5,
                   legendSize=10, figSizeLi=[5, 4], path='stackedbar.png', dpi=96)
    Visualization.DrawScatterPlot(protClustObj, currentFeatureSer, compareFeatureSer, color='Set1',
                    labelSize=10, path='scatterplot.png', figSizeLi=[5,4], dpi=96)


def Visualization_MetaColorMap():
    """
    Draw MetaColorMap that shows the distribution of each feature values in every clusters, the default is to choose all
    of the features, you can also use a list or csv file to choose which features you want to put in.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean', )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    ###draw MetaColorMap, show each samples' metadata feature

    labelLi = ['Gender', 'pathology_T_stage', 'pathology_N_stage', 'Stage', 'Integrated.Phenotype', 'CIN']
    Visualization.MetaColorMap(protDataObj, protClustObj, selectFeatureSer=labelLi, path='meta_colorbar.png')


def Visualization_SilhouettePlot():
    """
    Draw sihouette plot to see each cluster distance silhouette score, to make sure the clustering result is reasonable.
    Need to calculate silhouette score first
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean', )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Scoring.CalcEveryScore(protDataObj, protClustObj)

    # draw sihouette plot
    Visualization.DrawSilhouettePlot(protDataObj, protClustObj, path='silhou.png')


def Visualization_SampleOverlap():
    """
    Draw a heat map about the sample overlap between two clustering results, to see each cluster is similar or not.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)

    # clustering result 1
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean' )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    # clustering result 2
    protClustObj2 = ds_ProtClust()
    protClustObj2.clustParamObj.SetParam(method="hierarchicalClust", linkage="complete",distance="cosine", clustNum=5)
    protClustObj2 = ClusteringMethod.Clustering(protDataObj, protClustObj2)

    print('ARI: '+str(adjusted_rand_score(protClustObj.clustResultObj.sampleClustSer.tolist(),
                                  protClustObj2.clustResultObj.sampleClustSer.tolist())))

    Visualization.FindSampleOverlapHeatmap(protClustObj, protClustObj2, path='SampleOverlap.png')

def Visualization_Heatmap():
    """
        Draw a heat map of ratio table, you can sort the data first.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)

    # sort protein data
    protDataObj = DataAdapter.SortByRow(protDataObj)


    # clustering
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean' )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Visualization.DrawHeatMap(protDataObj, protClustObj, colorStr="coolwarm", clustColorStr='Set1', wSpace=0.02, figSizeLi=[20, 20],
                path=None, dpi=96)

def Visualization_CorrelationHeatmap():

    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean' )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    #Scoring.CalcEveryScore(protDataObj, protClustObj)
    Visualization.DrawPatientToPatientCorrHeatmap(protDataObj, protClustObj, path="corHeatmap.png",
                                                  colorsStr="coolwarm", figSizeLi=[10, 10], dpi=96)


def Visualization_PCA():
    """
    Draw a PCA plot to see the clustering result in a 2D plot.

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean' )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Visualization.PlotPca(protDataObj, protClustObj, dpi=96, path='PCA.png')


def Visualization_TSNE():
    """
    Draw a TSNE plot to see the clustering result in a 2D plot.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean' )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Visualization.PlotTsne(protDataObj, protClustObj, perplexity=30, dpi=96, figSizeLi=[6.4, 4.8], pointSize=6,
                           path='TSNE.png')

def Visualization_MosaicPlot():
    """
        Draw a mosaic plot to see the relation of a feature and clustering result .

    """
    protDataObj = DataAdapter.ReadFilesPath("Example/pancan32/pancan11_RGB/pancan11protRatio.csv",
                                            "Example/pancan32/pancan11_RGB/pancan11metadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=8, maxK=8, resample=0.8, resample_times=1000, ccDistance='manhattan')

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Scoring.CalcFeatureAriScore(protDataObj, protClustObj)
    Visualization.DrawMosaicPlot(protDataObj, protClustObj, 'Cancer_Type', path='Mosaic.png')
Visualization_MosaicPlot()
### Statistics ###

def Statistics_Wilcon():
    """
    Run rank sum test to each cluster to see the p-value of each feature. the result can then be used to GO term and
    pathway analysis
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean' )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)
    clustNum = 5
    Statistics.RunWilconTest(protDataObj, protClustObj, clustNum, 'consensusClust')


def Statistics_GSEA():
    """
    Turn the ratio table and the clustering result into a gct and a cls file(GSEA format)
    :return:
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)

    Statistics.GenerateGctFile(protDataObj, path='GSEAGeneTable.gct')

    protClustObj = ds_ProtClust()

    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean')

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Statistics.GenerateClsFile(protClustObj, path='GSEASampleClust.cls')



def Characterization_Characterize():
    """
    Compare odd ratio between each cluster and feature.
    Need to run scoring function first.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean', )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)

    Scoring.CalcEveryScore(protDataObj, protClustObj)
    cutoffUpper = 2.0
    cutoffLower = 0.5
    Characterization.Characterize(protClustObj, cutoffUpper, cutoffLower)


def DumpObject():
    """
    After you finish clustering, you can save ds_ProtClust and ds_InputRawData object as pkl files to use next file.
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.RemoveProtWithNa(protDataObj)
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = ds_ProtClust()
    protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod='kmeansClust_nltk',
                                        minK=5, maxK=8, resample=0.8, resample_times=1000, ccDistance='euclidean', )

    protClustObj = ClusteringMethod.Clustering(protDataObj, protClustObj)
    DataAdapter.DumpRawDataObj(protDataObj, 'colonCancer.pkl')
    DataAdapter.DumpProtClustObj(protClustObj, 'colonCancer_consensus.pkl')


def LoadObject():
    """
    Load the pkl file into ds_ProtClust and ds_InputRawData object
    """
    protDataObj = DataAdapter.LoadRawDataObj('colonCancer.pkl')
    protClustObj = DataAdapter.LoadProtClustObj('colonCancer_consensus.pkl')

    print(protDataObj.GetProtRatioDf())
    print(protClustObj.clustResultObj.sampleClustSer)



import numpy as np
import pandas as pd
import copy
from .CommonFunction import GenerateStat, PrintStat


def FilterByRank(rawDataObj, topNum, methodStr='SD'):
    """
    Select topNum number of features.

    :param rawDataObj: ds_InputRawData 
    :type rawDataObj: object
    :param methodStr: 'SD' or 'MAD'
    :type methodStr: str
    :param topNum: Select topNum feature
    :type topNum: int
    :return: Return **ds_InputRawData** object without changing raw object
    :rtype: object
    """

    print('Before filtering:')
    PrintStat(rawDataObj)

    proteinRatioDf = rawDataObj.GetProtRatioDf()
    filterMethodSer = 'FilterByRank_'

    if methodStr == 'SD':
        filterMethodSer = filterMethodSer + 'SD_' + str(topNum)
        proteinRatioSD = np.std(proteinRatioDf, axis=0)
        filteredProtRatioDf = proteinRatioDf[proteinRatioSD.sort_values(ascending=False).head(topNum).index]
    elif methodStr == 'MAD':
        filterMethodSer = filterMethodSer + 'MAD_' + str(topNum)
        proteinRatioMAD = proteinRatioDf.apply(lambda x: x.mad())
        filteredProtRatioDf = proteinRatioDf[proteinRatioMAD.sort_values(ascending=False).head(topNum).index]
    else:
        print('Error: method is either \'SD\' or \'MAD\'.')
        filterMethodSer = ''
        filteredProtRatioDf = pd.DataFrame()

    ds_FilterDataObj = copy.deepcopy(rawDataObj)
    ds_FilterDataObj.SetProtRatioDf(protRatioDf=filteredProtRatioDf)
    ds_FilterDataObj.filterMethodStr = filterMethodSer

    print('After filtering')
    ds_FilterDataObj = GenerateStat(ds_FilterDataObj)
    print('Filter method: ' + filterMethodSer + '\n')

    return ds_FilterDataObj


def FilterByCutoff(rawDataObj, cutOff, methodStr='SD'):
    """
    Select features that SD or MAD are higher than **cutOff**.

    :param rawDataObj: ds_InputRawData 
    :type rawDataObj: object
    :param methodStr: 'SD' or 'MAD'
    :type methodStr: str
    :param cutOff: Select feature that SD or MAD are higher than this.
    :type cutOff: int
    :return: Return **ds_InputRawData** object without changing raw object
    :rtype: object
    """

    print('Before filtering:')
    PrintStat(rawDataObj)

    proteinRatioDf = rawDataObj.GetProtRatioDf()
    filterMethodStr = 'FilterByCutoff_'

    if methodStr == 'SD':
        filterMethodStr = filterMethodStr + 'SD_' + str(cutOff)
        proteinRatioSD = np.std(proteinRatioDf, axis=0)
        filteredProtRatioDf = proteinRatioDf[proteinRatioSD[proteinRatioSD > cutOff].index]
    elif methodStr == 'MAD':
        filterMethodStr = filterMethodStr + 'MAD_' + str(cutOff)
        proteinRatioMAD = proteinRatioDf.apply(lambda x: x.mad())
        filteredProtRatioDf = proteinRatioDf[proteinRatioMAD[proteinRatioMAD > cutOff].index]
    else:
        print('Error: method is either \'SD\' or \'MAD\'.')
        filterMethodStr = ''
        filteredProtRatioDf = pd.DataFrame()

    ds_FilterDataObj = copy.deepcopy(rawDataObj)
    ds_FilterDataObj.SetProtRatioDf(protRatioDf=filteredProtRatioDf)
    ds_FilterDataObj.filterMethodStr = filterMethodStr

    print('After filtering:')
    ds_FilterDataObj = GenerateStat(ds_FilterDataObj)
    print('Filter method: ' + filterMethodStr + '\n')

    return ds_FilterDataObj


def FilterByTopPercentage(rawDataObj, topPercentage, methodStr='SD'):
    """
    Select topPercentage(float) of features.

    :param rawDataObj: ds_InputRawData
    :type rawDataObj: object
    :param methodStr:  'SD' or 'MAD'
    :type methodStr: str
    :param topPercentage:  0 <= topPercentage <= 1
    :type topPercentage: float
    :return: Return **ds_InputRawData** object without changing raw object
    :rtype: object
    """

    print('Before filtering:')
    PrintStat(rawDataObj)

    proteinRatioDf = rawDataObj.GetProtRatioDf()
    filterMethodStr = 'FilterByTopPercentage_'
    topNum = int(topPercentage * proteinRatioDf.shape[1])

    if methodStr == 'SD':
        filterMethodStr = filterMethodStr + 'SD_' + str(topPercentage)
        proteinRatioSD = np.std(proteinRatioDf, axis=0)
        filteredProtRatioDf = proteinRatioDf[proteinRatioSD.sort_values(ascending=False).head(topNum).index]
    elif methodStr == 'MAD':
        filterMethodStr = filterMethodStr + 'MAD_' + str(topPercentage)
        proteinRatioMAD = proteinRatioDf.apply(lambda x: x.mad())
        filteredProtRatioDf = proteinRatioDf[proteinRatioMAD.sort_values(ascending=False).head(topNum).index]
    else:
        print('Error: method is either \'SD\' or \'MAD\'.')
        filterMethodStr = ''
        filteredProtRatioDf = pd.DataFrame()

    ds_FilterDataObj = copy.deepcopy(rawDataObj)
    ds_FilterDataObj.SetProtRatioDf(protRatioDf=filteredProtRatioDf)
    ds_FilterDataObj.filterMethodStr = filterMethodStr

    print('After filtering:')
    ds_FilterDataObj = GenerateStat(ds_FilterDataObj)
    print('Filter method: ' + filterMethodStr + '\n')

    return ds_FilterDataObj


def FilterByGeneList(rawDataObj, geneLi):
    """
    Select features in geneList.

    :param rawDataObj: ds_InputRawData
    :type rawDataObj: object
    :param geneLi: A list of gene name.
    :type geneLi: list
    :return: Return **ds_InputRawData** object without changing raw object
    :rtype: object
    """

    print('Before filtering:')
    PrintStat(rawDataObj)

    proteinRatioDf = rawDataObj.GetProtRatioDf()
    filterMethodStr = 'FilterByGeneList'

    if geneLi == '' or geneLi == np.nan:
       print('Error.\nGeneList cannot be null.\n')
       return 0

    elif all([gene in list(proteinRatioDf.columns) for gene in geneLi]) and geneLi != '':
        filteredProtRatioDf = proteinRatioDf[geneLi]

    else:
        print('Error.\nGene in geneList must be involved in protein ratio.\n')
        return 0

    ds_FilterDataObj = copy.deepcopy(rawDataObj)
    ds_FilterDataObj.SetProtRatioDf(protRatioDf=filteredProtRatioDf)
    ds_FilterDataObj.filterMethodStr = filterMethodStr

    print('After filtering:')
    ds_FilterDataObj = GenerateStat(ds_FilterDataObj)
    print('Filter method: ' + filterMethodStr + '\n')

    return ds_FilterDataObj


import Scoring
import ClusteringMethod
import Characterization
import DataAdapter
import Visualization  
from DataStructure import ds_InputRawData
from DataStructure import ds_ProtClust
import time
def ClusteringTest():
	# rawDataObj = DataAdapter.LoadRawDataObj('./rawData.pickle')
	# protClustObj = DataAdapter.LoadProtClustObj('./protClust.pickle')
	# print(protClustObj.clustResultObj.characterRatioDf)
	rawDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv","Example/colonCancerMetadata.csv")
	# rawDataObj = DataAdapter.ReadFilesPath("~/Desktop/pancan11protRatio.csv","~/Desktop/pancan11metadata.csv")
	protClustObj = ds_ProtClust()

	# protClustObj.clustParamObj.SetParam(method='kmeansClust_nltk',clustNum=5,kmeansDisatance='pcc',repeats=10)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# protClustObj = Scoring.CalcEveryScore(rawDataObj,protClustObj)
	# Characterization.PrintParagraph(rawDataObj,protClustObj)

	# protClustObj.clustParamObj.SetParam(method='kmeansClust_nltk',clustNum=5,kmeansDisatance='euclidean',repeats=1)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)

	# protClustObj.clustParamObj.SetParam(method='kmeansClust_nltk',clustNum=5,kmeansDisatance='euclidean',repeats=10)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)

	# protClustObj = Characterization.Characterize(protClustObj,2,0.5)
	# Characterization.PrintParagraph(rawDataObj,protClustObj)

	# protClustObj.clustParamObj.SetParam(method='kmeansClust_sklearn',clustNum=5,seed=10)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)
	# protClustObj = Characterization.Characterize(protClustObj,2,0.5)
	#
	# protClustObj.clustParamObj.SetParam(method='hierarchicalClust_scipy',clustNum=5,linkage='ward',distance='cosine')
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)
	# protClustObj = Characterization.Characterize(protClustObj,2,0.5)
	
	# protClustObj.clustParamObj.SetParam(method='hierarchicalClust',clustNum=5,linkage='average',distance='euclidean')
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)
	# protClustObj = Characterization.Characterize(protClustObj,2,0.5)
	
	# protClustObj.clustParamObj.SetParam(method='consensusClust',ccMethod='hierarchicalClust',resample=0.8,ccLinkage='ward',ccDistance='euclidean',minK=4,maxK=6,resample_times=10)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# Visualization.DrawConsensusClustCDF(protClustObj)

	protClustObj.clustParamObj.SetParam(method='consensusClust', ccMethod='hierarchicalClust', resample=0.8,
										ccLinkage='single',ccFinalLinkage='average', ccDistance='euclidean',ccFinalDistance='pcc',
										minK=4, maxK=6, resample_times=10)
	protClustObj = ClusteringMethod.Clustering(rawDataObj, protClustObj)
	# Visualization.DrawConsensusClustCDF(protClustObj)

	# protClustObj.clustParamObj.SetParam(method='consensusClust', ccMethod='hierarchicalClust', resample=0.8,
	# 									ccLinkage='average',ccDistance='euclidean', minK=4, maxK=6, resample_times=10)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj, protClustObj)
	# Visualization.DrawConsensusClustCDF(protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)
	# protClustObj = Characterization.Characterize(protClustObj, 2, 0.5)

	protClustObj.clustParamObj.SetParam(method='consensusClust',ccMethod='hierarchicalClust_scipy',resample=0.8,ccLinkage='ward',ccDistance='cosine',ccFinalDistance='correlation',minK=4,maxK=6,resample_times=10)
	protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	# Visualization.DrawConsensusClustCDF(protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)
	# protClustObj = Characterization.Characterize(protClustObj,2,0.5)
	#
	# protClustObj.clustParamObj.SetParam(method='consensusClust',ccMethod='kmeansClust_nltk',resample=0.8,ccDistance='pcc',minK=2,maxK=6,resample_times=10)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
	#
	# protClustObj.clustParamObj.SetParam(method='consensusClust', ccMethod='kmeansClust_sklearn', resample=0.8,
	# 									 minK=2, maxK=6, resample_times=10)
	# protClustObj = ClusteringMethod.Clustering(rawDataObj, protClustObj)
	# Visualization.DrawConsensusClustCDF(protClustObj)
	# protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)
	# protClustObj = Characterization.Characterize(protClustObj,2,0.5)

	# DataAdapter.DumpRawDataObj(rawDataObj,'./rawData.pickle')
	# DataAdapter.DumpProtClustObj(protClustObj,'./protClust.pickle')

def ReadClustResult(path):
	import pandas as pd
	rawDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
	protClustObj = ds_ProtClust()
	input_file = pd.read_csv(path,header=None)
	protClustObj = ClusteringMethod.ReadClustSeries(input_file[1],rawDataObj,protClustObj)
	protClustObj = Scoring.CalcFeatureRatio(rawDataObj,protClustObj)
	protClustObj = Characterization.Characterize(protClustObj,2,0.5)
	protClustObj = Characterization.Characterize(protClustObj,3/2,2/3)

def Kmeans_test():
	from sklearn.metrics.cluster import adjusted_rand_score
	import numpy as np
	from scipy import stats
	rawDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv","Example/colonCancerMetadata.csv")
	protClustObj = ds_ProtClust()

	nltk_kmeans = []
	sklearn_kmeans = []
	nltk_kmeans_ari = []
	sklearn_kmeans_ari = []
	hclust = []
	hclust_ari = []
	ccMatrix = []
	ccMatrix_ari = []

	for i in range(100):
		protClustObj.clustParamObj.SetParam(method='consensusClust',ccMethod='kmeansClust_sklearn',resample=0.8,minK=5,maxK=5,resample_times=1000)
		protClustObj,mk = ClusteringMethod.Clustering(rawDataObj,protClustObj)
		mk = np.array(mk)
		mk = mk.flatten()
		# sklearn_kmeans.append(protClustObj.clustResultObj.sampleClustSer)
		sklearn_kmeans.append(mk)

		protClustObj.clustParamObj.SetParam(method='consensusClust',ccMethod='kmeansClust_nltk',resample=0.8,ccDistance='euclidean',minK=5,maxK=5,resample_times=1000)
		protClustObj,mk = ClusteringMethod.Clustering(rawDataObj,protClustObj)
		# nltk_kmeans.append(protClustObj.clustResultObj.sampleClustSer)
		mk = np.array(mk)
		mk = mk.flatten()
		nltk_kmeans.append(mk)

		# protClustObj.clustParamObj.SetParam(method='consensusClust',ccMethod='hierarchicalClust',resample=0.8,ccLinkage='ward',ccDistance='euclidean',minK=5,maxK=6,resample_times=1)
		# protClustObj,mk = ClusteringMethod.Clustering(rawDataObj,protClustObj)
		# # hclust.append(protClustObj.clustResultObj.sampleClustSer)
		# mk = np.array(mk)
		# mk = mk.flatten()
		# hclust.append(mk)
	
	for i in range(100):
		for j in range(100):
			ccMatrix =  round(stats.pearsonr(sklearn_kmeans[i], nltk_kmeans[j])[0],2)
			# ccMatrix = round(adjusted_rand_score(sklearn_kmeans[i],nltk_kmeans[j]),2)
			ccMatrix_ari.append(ccMatrix)
	# for i in range(10):
	# 	for j in range(i+1,10):

	# 		# sklean_ari = round(adjusted_rand_score(sklearn_kmeans[i],sklearn_kmeans[j]),2)
	# 		# nltk_ari = round(adjusted_rand_score(nltk_kmeans[i],nltk_kmeans[j]),2)
	# 		# sklearn_kmeans_ari.append(sklean_ari)
	# 		# nltk_kmeans_ari.append(nltk_ari)
	# 		h_ari = round(adjusted_rand_score(hclust[i],hclust[j]),2)
	# 		hclust_ari.append(h_ari)
	# 		# print("Cluster 1 : {}, Cluster 2 : {} , ARI score : {}".format(i,j,ari))
	# print(sklearn_kmeans_ari)
	# print('\n\n\n')
	# print(nltk_kmeans_ari)

	with open('./kmeans_ari2.txt','w') as of:
		# of.write("sklearn kmeans ari :\n")
		# of.write(str(sklearn_kmeans_ari))
		# of.write("nltk kmeans ari :\n")
		# of.write(str(nltk_kmeans_ari))
		of.write("sklearn & nltk consensus matrix ari :\n")
		of.write(str(ccMatrix_ari))
	of.close()
	
def hierarchicalClustTest():

	from sklearn.metrics.cluster import adjusted_rand_score
	import numpy as np
	from scipy import stats
	rawDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv","Example/colonCancerMetadata.csv")
	protClustObj = ds_ProtClust()

	scipy_distance = ['euclidean','correlation','cosine','cityblock']
	sklearn_distance = ['euclidean','pcc','cosine','manhattan']
	linkage = ['ward','complete','average','single']
	ari = []
	for i in range(4):
		if(linkage[i] == 'ward'):
				protClustObj.clustParamObj.SetParam(method='hierarchicalClust',clustNum=5,linkage=linkage[i],distance='euclidean')
				protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
				a = protClustObj.clustResultObj.sampleClustSer
				p = "ward_euclidean_hierarchicalClust"
				Visualization.PlotPca(rawDataObj,protClustObj,path=p)
				protClustObj.clustParamObj.SetParam(method='hierarchicalClust_scipy',clustNum=5,linkage=linkage[i],distance='euclidean')
				protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
				b = protClustObj.clustResultObj.sampleClustSer
				p = "ward_euclidean_hierarchicalClust_scipy"
				colorLi = ['#2ca02c','#d62728','#ff7f0e','#9467bd','#1f77b4']
				Visualization.PlotPca(rawDataObj,protClustObj,path=p,colorLi=colorLi)
				ari.append(round(adjusted_rand_score(a,b),2))
		# else:
		# 	for j in range(4):
		# 			protClustObj.clustParamObj.SetParam(method='hierarchicalClust_scipy',clustNum=5,linkage=linkage[i],distance=scipy_distance[j])
		# 			protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
		# 			a = protClustObj.clustResultObj.sampleClustSer
		# 			p = linkage[i]+'_'+scipy_distance[j]+"hierarchicalClust_scipy"
		# 			Visualization.PlotPca(rawDataObj,protClustObj,path=p)
		# 			protClustObj.clustParamObj.SetParam(method='hierarchicalClust',clustNum=5,linkage=linkage[i],distance=sklearn_distance[j])
		# 			protClustObj = ClusteringMethod.Clustering(rawDataObj,protClustObj)
		# 			b = protClustObj.clustResultObj.sampleClustSer
		# 			p = linkage[i]+'_'+sklearn_distance[j]+"_hierarchicalClust"
		# 			Visualization.PlotPca(rawDataObj,protClustObj,path=p)
		# 			ari.append(round(adjusted_rand_score(a,b),2))
	print(ari)

def consensusKMTest():
	from sklearn.metrics.cluster import adjusted_rand_score
	rawDataObj1 = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
	rawDataObj2 = DataAdapter.ReadFilesPath("~/Desktop/pancan11protRatio.csv","~/Desktop/pancan11metadata.csv")
	protClustObj = ds_ProtClust()

	ari_list = []
	# dist = 'euclidean'
	# dist = 'pcc'
	# dist = 'cosine'
	# dist = 'manhattan'
	dist = ['euclidean','pcc','cosine','manhattan']
	for i in range(4):

		protClustObj.clustParamObj.SetParam(method='consensusClust', ccMethod='kmeansClust_nltk', resample=0.8,
											ccDistance=dist[i], minK=5, maxK=5, resample_times=100)
		protClustObj = ClusteringMethod.Clustering(rawDataObj1, protClustObj)
		a = protClustObj.clustResultObj.sampleClustSer
		ari_list.append(a)
	nltk_kmeans_ari = []
	for i in range(4):
		for j in range(i+1,4):
			nltk_ari = round(adjusted_rand_score(ari_list[i],ari_list[j]),2)
			nltk_kmeans_ari.append(nltk_ari)
	print(nltk_kmeans_ari)




ClusteringTest()
# ReadClustResult('~/Downloads/colon_ward_hierarchical.csv')
# Kmeans_test()
# hierarchicalClustTest()
# consensusKMTest()



from .CommonFunction import *


from sklearn.metrics import adjusted_rand_score

def CalcFeatureAriScore(rawDataObj, protClustObj):
    """
    傳入inputRawData和protClust、beta及clustering結果，計算各個metadata feauture的ARI並儲存至ds_protClust object中的featureVscoreDi dictionary

    :param rawDataObj: ds_inputRawData object
    :param protClustObj: ds_protClust object
    :return: ds_protClust object
    """

    metaColName = rawDataObj.GetMetadataDf().columns.values
    metaDf = rawDataObj.GetMetadataDf().copy()
    predLi = protClustObj.clustResultObj.sampleClustSer.values.copy()

    protClustObj.clustScoringObj.featureAriDi = dict()
    for colName in metaColName:
        
        trueLi = TurnLabelToNum(metaDf[colName])

        predLi = TurnLabelToNum(predLi)

        protClustObj.clustScoringObj.featureAriDi.update({colName: adjusted_rand_score(trueLi, predLi)})
    return protClustObj



from sklearn.mixture import GaussianMixture


def GaussianEMClust(rawDataObj, protClustObj):
    """

    :param rawDataObj: ds_InputRawData
    :type rawDataObj: object 
    :param protClustObj: ds_ProtClust
    :type protClustObj: object
    :returns: predArr 

    .. note::
        The parameter in protClustObj.clustParamObj.__kwargs must include 'clustNum', 'covType'
    """
    clustNum = protClustObj.clustParamObj.GetParam('clustNum')
    covType = protClustObj.clustParamObj.GetParam('covType')
    dataArr = rawDataObj.GetProtRatioDf().values

    gaussianClusterer = GaussianMixture(n_components=clustNum, covariance_type=covType)
    predArr = gaussianClusterer.fit_predict(X=dataArr)

    return predArr



import setuptools

REQUIRES = ["numpy"]

with open("README.md", "r") as fh:
    long_description = fh.read()
PACKAGES = ['ProteomeClustering',
'ProteomeClustering.CaseStudy',
'ProteomeClustering.Characterization',
'ProteomeClustering.ClusteringMethod',
'ProteomeClustering.DataAdapter',
'ProteomeClustering.DataStructure',
'ProteomeClustering.Example',
'ProteomeClustering.Scoring',
'ProteomeClustering.Statistics',
'ProteomeClustering.Visualization']

setuptools.setup(
    name="proteomeClusteringtest2", # Replace with your own username
    version="0.0.7",
    author="Tse-Ming Chen",
    author_email="benben5514@gmail.com",
    description="proteomeClustering test",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zytyz/ProteomeClustering",
    project_urls={
            "Documentation": "https://pillow.readthedocs.io" },
    packages=PACKAGES,
    install_requires=REQUIRES,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



from sklearn.metrics import adjusted_rand_score

from .CommonFunction import *
import seaborn as sns
import numpy as np
import matplotlib.pyplot as plt
import scipy as sc
import matplotlib.colors
from scipy.spatial import distance
import math

def DrawHeatMap(rawDataObj, protClustObj, colorStr="coolwarm", clustColorStr='Set1', wSpace=0.02, figSizeLi=[20, 20],
                path=None, dpi=96):
    """
    draw a heat map based on rawDataObj filtered ratio table

    :param rawDataObj: ds_InputRawData
    :param protClustObj: ds_ProtClust
    :param colorStr: the matplotlib color set used in heatmap, default is "coolwarm"
    :param clustColorStr: (if hierarchical clustering) the matplotlib color set used in clustermap to show different subtypes, defalt is "Set1"
    :param wSpace: (if not hierarchical clustering) the width space between each subtype
    :param figSizeLi: the figure size of the heatmap
    :param path: the path to save heatmap figure
    :return: heatmap fig
    """

    clustNumSer = protClustObj.clustResultObj.sampleClustSer

    heatMapRatioDf = rawDataObj.GetProtRatioDf().copy()
    heatMapRatioDf['clusterNumList'] = clustNumSer

    lenEachClustLi = GetlenEachClustLi(protClustObj)

    ###sort same cluster together###
    heatMapRatioDf.sort_values(by='clusterNumList', axis=0, inplace=True)

    heatMapRatioDf = heatMapRatioDf.drop(columns=['clusterNumList'])
    sampleNameli = heatMapRatioDf.index.values
    # heatMapRatioDf = heatMapRatioDf[heatMapRatioDf.columns != 'clusterNumList']

    methodStr = protClustObj.clustParamObj.GetParam('method')

    if methodStr == 'hierarchicalClust':

        pltColors = sns.color_palette(clustColorStr, len(set(clustNumSer)))
        cmap = matplotlib.colors.ListedColormap(pltColors)
        rowClusterColor = clustNumSer.map(cmap)
        metricStr = protClustObj.clustParamObj.GetParam('distance')
        linkageStr = protClustObj.clustParamObj.GetParam('linkage')

        fig = sns.clustermap(heatMapRatioDf, row_colors=rowClusterColor, metric=metricStr, method=linkageStr,
                                   cmap=colorStr, xticklabels=False, yticklabels=False,
                                   figsize=(figSizeLi[0], figSizeLi[1]))
        ax = fig.ax_heatmap
        ax.set_xlabel("gene")
        ax.set_ylabel("sample")

    else:
        # print('only hierarchical clustering can show heatmap')

        ### draw heatmap
        fig, axes = plt.subplots(1, len(lenEachClustLi), figsize=(figSizeLi[0], figSizeLi[1]),
                                 gridspec_kw={'hspace': 0, 'wspace': wSpace, 'width_ratios': lenEachClustLi})
        heatMapStart = 0
        heatMapEnd = 0
        cbarAx = fig.add_axes([.905, .3, .05, .3])
        for i in range(len(lenEachClustLi)):
            heatMapEnd += lenEachClustLi[i]
            if i == len(lenEachClustLi) - 1:
                # (__SortSampleRatioDf(heatMapRatioDf.iloc[:, heatMapStart:heatMapEnd],sampleNameli[heatMapStart:heatMapEnd])

                sns.heatmap(heatMapRatioDf.T.iloc[:, heatMapStart:heatMapEnd],
                            ax=axes[i], xticklabels=False, yticklabels=False, cmap=colorStr, robust=True, cbar_ax=cbarAx)
            else:
                sns.heatmap(heatMapRatioDf.T.iloc[:, heatMapStart:heatMapEnd],
                            ax=axes[i], cbar=False, xticklabels=False, yticklabels=False, cmap=colorStr, robust=True)
            heatMapStart += lenEachClustLi[i]
            axes[i].set_xlabel('cluster' + str(i + 1))

        # confiugration: figure_size, color?, minmax, center,

    if path == None:
        plt.show()
    else:
        plt.savefig(path, dpi=dpi)
    return fig


def DrawPatientToPatientCorrHeatmap(rawDataObj, protClustObj, colorsStr="coolwarm", figSizeLi=[10, 10],
                                    path=None, dpi=96):
    """
    draw a heat map based on sample to sample correlation

    :param rawDataObj: ds_InputRawData
    :param protClustObj: ds_ProtClust
    :param colorsStr: the matplotlib color set used in heatmap, default is "coolwarm"
    :param figSizeLi: the figure size of the heatmap
    :param path: the path to save heatmap figure
    :return: heatmap fig
    """
    sampleNameli = rawDataObj.GetProtRatioDf().index.values
    distArr = [[0.0 for i in range(len(sampleNameli))] for k in range(len(sampleNameli))]
    clustNumSer = protClustObj.clustResultObj.sampleClustSer

    heatMapRatioDf = rawDataObj.GetProtRatioDf().copy()
    heatMapRatioDf['clusterNumList'] = clustNumSer

    ###sort same cluster together###
    heatMapRatioDf.sort_values(by='clusterNumList', axis=0, inplace=True)

    heatMapRatioDf = heatMapRatioDf.drop(columns=['clusterNumList'])
    sampleNameli = heatMapRatioDf.index.values
    col = 0

    for name1 in sampleNameli:
        row = 0
        for name2 in sampleNameli:
            distArr[col][row] = sc.spatial.distance.euclidean(heatMapRatioDf.T[name1], heatMapRatioDf.T[name2])
            #if distArr[col][row]!=0:
            #    distArr[col][row] = math.log2(distArr[col][row])
            row += 1
        col += 1
    plt.figure(figsize=(figSizeLi[0], figSizeLi[1]))

    sns.heatmap(distArr, xticklabels=False, yticklabels=False, cmap=colorsStr, robust=True, square=True,vmin=0)
    print(distArr)
    if path is None:
        plt.show()
    else:
        plt.savefig(path, dpi=dpi)
    return plt



import pandas as pd
import math
from collections import Counter


def GetlenEachClustLi(protClustObj):
    """
    used in heatmap and metacolormap plot, get the sample number of each cluster

    :param protClustObj:
    :return: int list, for example, [20, 20, 20, 20, 16] for 5 clsuters and 96 samples
    """

    clustSampleLi = list(protClustObj.clustResultObj.clustSampleSer)
    lenLi = [0 for x in range(len(clustSampleLi))]
    for col in range(len(clustSampleLi)):
        lenLi[col] = len(clustSampleLi[col])

    return lenLi


def CountFeatureValueNum(metaSer, featureValue):
    """
    count how many a feature value(input) in the series

    :param metaSer: the meata data feature series in specific cluster
    :param featureValue: the feature value that wanted to count
    :return: int, number of the featureValue
    """
    count = 0
    for i in range(len(metaSer)):
        if metaSer[i] == featureValue:
            count = count + 1
    return count


def GetXlabelPositionLi(sampleClustSer):
    """
    Get the x label position to draw diverge and stacked plot

    :param sampleClustSer: series, show each sample belongs to which cluster
    :return: x label position list
    """
    xLabelLi = []
    for index in range(len(set(sampleClustSer))):
        xLabelLi.append(index)

    return xLabelLi


def GetEntropy(featureLi, unit='shannon'):
    """
    compute entropy of data(meta data feature)

    :param featureLi: label list or number list, len is same as sample numbers
    :param unit: the log base of computing entropy. shannon=2, natural=e, hartley=10
    :return: float entropy
    """

    base = {
        'shannon': 2.,
        'natural': math.exp(1),
        'hartley': 10.
    }

    if len(featureLi) <= 1:
        return 0

    counts = Counter()

    for d in featureLi:
        counts[d] += 1

    ent = 0

    probs = [float(c) / len(featureLi) for c in counts.values()]
    for p in probs:
        if p > 0.:
            ent -= p * math.log(p, base[unit])

    return ent


def GetClustXaxisLi(clustNumSer):
    """
    get the x axis label list used in stacked, scatter, diverge plot (cluster 1, 2, 3, 4, 5...)

    :param clustNumSer:
    :return: string list,
    """
    clustXaxisLi = []
    #####build index list of entropy table
    for index in set(clustNumSer):
        # clustXaxisLi.append("cluster" + str(index))
        clustXaxisLi.append(str(index))
    return clustXaxisLi



from .CommonFunction import *


def CalcPurityScore(rawDataObj, protClustObj):
    """
    :param rawDataObj: ds_inputRawData object
    :param protClustObj: ds_protClust object
    :return: ds_protClust object
    """
    metaDf = rawDataObj.GetMetadataDf().copy()
    predLi = np.array(protClustObj.clustResultObj.sampleClustSer).copy()
    purity = 0
    for clusIdx in range(1, np.max(predLi) + 1):
        boolMask = (predLi == clusIdx)
        maxClusterPurity = 0

        for featureName in metaDf.columns:
            clusterPurity = _ChooseLargestFeature(metaDf[featureName][boolMask].value_counts())
            if clusterPurity > maxClusterPurity:
                maxClusterPurity = clusterPurity
        purity += maxClusterPurity

    purity /= metaDf.shape[0]
    protClustObj.clustScoringObj.purity = purity

    return protClustObj


def _ChooseLargestFeature(featureSer):
    tempNum = 0
    result = 0
    featureSer = featureSer.rename(index=str)
    for name in featureSer.index.values:
        if float(featureSer[name]) > tempNum:
            tempNum = float(featureSer[name])
            result = float(featureSer[name])
    return result


from .Vscore import CalcFeatureVscore
from .Fscore import CalcFeatureMaxFscore
from .ARIscore import CalcFeatureAriScore
from .AMIscore import CalcFeatureAmiScore
from .PairedFscore import CalcFeaturePairedFscore
from .SilhouetteScore import CalcSilhScore, CalcSampleSilhScore, CalcClustSilhScore
from .PartitionIndex import CalcPartitionIndex
from .SeparationIndex import CalcSeparationIndex
from .GiniScore import CalcGiniScore
from .PurityScore import CalcPurityScore
from .FeatureRatio import CalcFeatureRatio
from .FeatureEntropy import CalcClustFeatureEntropy, CalcFeatureEntropy
from .BICscore import CalcBicScore


def CalcEveryScore(rawDataObj, protClustObj):
    '''
    Calculates all available scores and store them in protClustObj

    :param rawDataObj:
    :param protClustObj:
    :return:
    '''
    scoreOptionsLi = [CalcFeatureVscore, CalcFeatureMaxFscore, CalcFeatureAriScore, CalcFeatureAmiScore,
                      CalcFeaturePairedFscore, CalcSilhScore, CalcSampleSilhScore, CalcClustSilhScore,
                      CalcPartitionIndex,
                      CalcSeparationIndex, CalcGiniScore, CalcPurityScore, CalcFeatureRatio, CalcClustFeatureEntropy,
                      CalcFeatureEntropy,
                      CalcBicScore]

    for scoreOption in scoreOptionsLi:
        protClustObj = scoreOption(rawDataObj, protClustObj)

    return protClustObj



import numpy as np
import pandas as pd
from ProteomeClustering import DataAdapter
from ProteomeClustering import ClusteringMethod
from ProteomeClustering import Characterization
from ProteomeClustering import Statistics
from ProteomeClustering import Visualization
from ProteomeClustering import CaseStudy
from ProteomeClustering import Scoring
from ProteomeClustering.DataStructure import ds_InputRawData
from ProteomeClustering.DataStructure import ds_ProtClust
from scipy import stats
from sklearn.metrics import adjusted_rand_score
from sklearn.metrics.cluster import v_measure_score

import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from sklearn import metrics
from pandas.core.frame import DataFrame
import random


def MakeClusterResultCsvIntoObject(path='../../pathway_ontology_analysis/clustering_data/differentfilter_4/filter1.csv'):
    '''
    use csv file as a clustering result
    :param path:
    :return:
    '''
    protClustObj = ds_ProtClust()

    protClustObj.clustResultObj.sampleClustSer = pd.read_csv(path, header=None, index_col=0)

    protClustObj.clustResultObj.sampleClustSer = protClustObj.clustResultObj.sampleClustSer.iloc[:, 0]
    print(protClustObj.clustResultObj.sampleClustSer)
    return protClustObj

def SampleOverlap(resampleTimes=30):
    """
    find
    :param resampleTimes: the number to run clustering, to find the 5 similar clustering we want
    :return:
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    successCount = 0
    protClustObj = DataAdapter.LoadProtClustObj("consensusProtClust.pkl")
    protClustObj1 = [ds_ProtClust() for x in range(resampleTimes)]
    # protClustObj2 = [ds_ProtClust() for x in range(resampleTimes)]
    for num1 in range(resampleTimes):
        protClustObj1[num1].clustParamObj.SetParam(method="gaussianEMClust", clustNum=5,
                                                   covType='full')
        # protClustObj1[num1].clustParamObj.SetParam(method="hierarchicalClust", linkage="ward",
        #                                    distance="euclidean", clustNum=5)

        # protClustObj1[num1].clustParamObj.SetParam(method="consensusClust", ccMethod="kmeansClust_sklearn",
        #                                           clustNum=5, resample=0.8, minK=5, maxK=6, resample_times=10,
        #                                           ccDistance='euclidean')
        protClustObj1[num1] = ClusteringMethod.Clustering(protDataObj, protClustObj1[num1])

        print(adjusted_rand_score(protClustObj1[num1].clustResultObj.sampleClustSer.tolist(),
                                  protClustObj.clustResultObj.sampleClustSer.tolist()))

        ratioTable = Visualization.FindSampleOverlapHeatmap(protClustObj, protClustObj1[num1])
        xAxisLi = [x for x in range((len(ratioTable)))]
        yAxisLi = [x for x in range((len(ratioTable)))]

        if _IsPassOverLapBar(xAxisLi, yAxisLi, ratioTable) & _IsPassSampleNumBar(protClustObj1[num1]):
            successCount += 1
            sns.heatmap(ratioTable, cmap="Reds", linewidths=.5, annot=True, xticklabels=[x for x in set(xAxisLi)],
                        yticklabels=[x for x in set(yAxisLi)], vmin=0, vmax=1, cbar=False)
            plt.savefig("fig/successSampleOverlap" + str(successCount) + ".png")
            plt.cla()
            protClustObj1[num1].clustResultObj.sampleClustSer.to_csv("fig/PassOverLap" + str(successCount) + "_1.csv")
            DataAdapter.DumpProtClustObj(protClustObj1[num1], "fig/PassOverLap" + str(successCount) + "_1.pkl")
            # protClustObj2[num1].clustResultObj.sampleClustSer.to_csv("fig/PassOverLap" + str(successCount) + "_2.csv")
            # DataAdapter.DumpProtClustObj(protClustObj2[num1], "fig/PassOverLap" + str(successCount) + "_2.pkl")


def _IsPassOverLapBar(xAxisLi, yAxisLi, ratioTable, OverlapBar=0.4):
    """
    to test if each cluster has the similarity >overlapBar(default=0.3)
    :param xAxisLi:
    :param yAxisLi:
    :param ratioTable:
    :param OverlapBar:
    :return:
    """
    if not xAxisLi or not yAxisLi:
        return True
    sampleOverlapScore = 0
    tempI = -1
    tempK = -1
    for i in xAxisLi:

        for k in yAxisLi:
            if ratioTable[i][k] > sampleOverlapScore:
                sampleOverlapScore = ratioTable[i][k]
                tempI = i
                tempK = k
    ####
    if sampleOverlapScore <= OverlapBar:
        return False
    xAxisLi.remove(tempI)
    yAxisLi.remove(tempK)
    return _IsPassOverLapBar(xAxisLi, yAxisLi, ratioTable)


def _IsPassSampleNumBar(protClustObj, SampleNumBar=8):
    """
    to test if each cluster has enough samples(default>8)
    :param protClustObj:
    :param SampleNumBar:
    :return:
    """
    sampleLi1 = protClustObj.clustResultObj.sampleClustSer.tolist()
    for i in set(sampleLi1):
        count = 0
        for k in range(len(sampleLi1)):
            if sampleLi1[k] == i:
                count += 1
        print(count)
        if count < SampleNumBar:
            return False
    return True



def DifferentKmeansClusteringTest(resampleTimes=30,
                                  kmeanclusterfile="KmeansClusterFile.csv"):
    '''
    runt same clustering method multiple times to see if the clustering method is stable
    :param resampleTimes:
    :param kmeanclusterfile:
    :return:
    '''
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)


    ARITable = pd.DataFrame([[0.0 for x in range(resampleTimes)]] * resampleTimes)
    MaskTable = pd.DataFrame([[True for x in range(resampleTimes)]] * resampleTimes)
    logProtObj = [ds_ProtClust() for x in range(resampleTimes)]
    giniLi = [0.0 for x in range(resampleTimes)]

    gct = open(kmeanclusterfile, 'w')
    for num1 in range(resampleTimes):
        seed = random.randint(1, 100000)

        # logProtObj[num1].clustParamObj.SetParam(method="kmeansClust_sklearn", clustNum=5,
        #                                        seed=seed)
        #logProtObj[num1+15].clustParamObj.SetParam(method="gaussianEMClust", clustNum=5,
        #                                        covType='full')

        logProtObj[num1].clustParamObj.SetParam(method="consensusClust", ccMethod="kmeansClust_nltk",
                                                clustNum=5, resample=0.8, minK=5, maxK=6, resample_times=100,
                                                ccDistance='euclidean')

        logProtObj[num1] = ClusteringMethod.Clustering(protDataObj, logProtObj[num1])
        Scoring.CalcGiniScore(protDataObj, logProtObj[num1])
        giniLi[num1] = logProtObj[num1].clustScoringObj.giniScore
        gct.writelines(str(logProtObj[num1].clustResultObj.sampleClustSer.tolist()) + '\n')

    gct.close()

    ariScatterLi = []
    giniScatterLi = []
    histogramLi = []

    num2 = 0
    for num1 in range(resampleTimes):

        for num2 in range(num1):
            score = adjusted_rand_score(logProtObj[num1].clustResultObj.sampleClustSer.tolist()
                                        , logProtObj[num2].clustResultObj.sampleClustSer.tolist())
            ariScatterLi.append(score)
            giniScatterLi.append(_ChooseLargeGini(giniLi, num1, num2))
            ARITable[num1][num2] = score
            MaskTable[num1][num2] = False
            histogramLi.append(score)

    '''
    ### histogram of ARI and Gini Score
    sns.distplot(histogramLi, bins=5, hist_kws=dict(edgecolor="black", linewidth=2))
    plt.xlim(0, 1.0)
    plt.show()
    sns.distplot(giniLi, bins=5, hist_kws=dict(edgecolor="black", linewidth=2))
    plt.xlim(0, 1.0)
    plt.show()
    '''

    '''
    ### heatmap of resample times ari
    # fig, axes = plt.subplots(1, 1, figsize=(50, 50))
    # sns.heatmap(ARITable, cmap="Reds", linewidths=.5, annot=True, mask=MaskTable, vmin=0, vmax=1, ax=axes)
    # g = sns.heatmap(ARITable, cmap="Reds", linewidths=.5, annot=True, mask=MaskTable, vmin=0, vmax=1, fmt='.2f', xticklabels=1, yticklabels=1)
    # fig.savefig('differentkmeans.png', dpi=300)
    # g.set(xticklabels=[x for x in range(1, resampleTimes+1)],yticklabels=[x for x in range(1, resampleTimes+1)])
    # plt.show()
    # print(ARITable)
    '''
    '''
    ### scatter plot of gini and ARI Score
    fig, axes = plt.subplots(1, 1, dpi=300)
    scatterDataDf = pd.DataFrame(list(zip(ariScatterLi, giniScatterLi)), columns=['ARI', 'Average Gini Score'])
    sns.scatterplot(x='ARI', y='Average Gini Score', data=scatterDataDf, ax=axes)
    axes.set_xlim([0, 1])
    axes.set_ylim([0, 1])

    fig.savefig('Con100 30Scatter.png')
    '''

def _ChooseLargeGini(giniLi, num1, num2):
    '''
    if giniLi[num1]<= giniLi[num2]:
        return giniLi[num2]
    else:
        return giniLi[num1]
    to draw a scatter plot of gini and ari needs to calculate average gini score
    '''
    return (giniLi[num1] + giniLi[num2]) / 2


def DifferentFilterClusteringTest(filterLi=None, kmeanclusterfile="FilterClusterFile.csv"):
    """

    :param filterLi:
    :param kmeanclusterfile:
    :return:
    """

    if filterLi is None:
        filterLi = [
            # ['cutoff','MAD', 0.5], ['cutoff','MAD', 0.75],['cutoff','MAD',0.9],['cutoff','MAD', 1.0],
            # ['cutoff', 'SD',0.5], ['cutoff','SD', 0.75],['cutoff','SD',0.9],['cutoff','SD', 1.0],
            # ['rank','MAD', 400], ['rank','MAD', 600], ['rank','MAD', 800], ['rank','MAD', 1000], ['rank','MAD', 1200],
            ['rank', 'SD', 400], ['rank', 'SD', 600], ['rank', 'SD', 800], ['rank', 'SD', 1000], ['rank', 'SD', 1200]
        ]

    SD = ['400', '600', '800', '1000', '1200']

    geneNumDf = pd.DataFrame({"MAD": SD, 'geneNum': [0 for x in range(len(filterLi))]})

    rawDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")

    # print(nonlogDataObj.GetProtRatioDf(filter=1))

    ARITable = pd.DataFrame([[0.0 for x in range(len(filterLi))]] * len(filterLi))
    logProtObj = [ds_ProtClust() for x in range(len(filterLi))]
    MaskTable = pd.DataFrame([[True for x in range(len(filterLi))]] * len(filterLi))

    gct = open(kmeanclusterfile, 'w')
    num1 = 0
    for func, method, cutoff in filterLi:

        if func == 'cutoff':
            rawDataObj = DataAdapter.FilterByCutoff(rawDataObj, method=method, cutOff=cutoff)

        elif func == 'rank':
            rawDataObj = DataAdapter.FilterByRank(rawDataObj, method=method, topNum=cutoff)

        rawDataObj.GetProtRatioDf().to_csv(func + method + str(cutoff) + ".csv")

        logProtObj[num1].clustParamObj.SetParam(method="consensusClustering", ccMethod="kmeans",
                                                clustNum=5, resample=0.9, minK=5, maxK=9, resample_times=1000)

        logProtObj[num1] = ClusteringMethod.clustering(rawDataObj, logProtObj[num1])
        geneNumDf['geneNum'][num1] = len(rawDataObj.GetProtRatioDf().columns.values)

        gct.writelines(str(logProtObj[num1].clustResultObj.sampleClustSer.tolist()) + '\n')

        num1 += 1

    gct.close()

    smallScore = 100.0
    number1 = 0
    number2 = 0
    histogramLi = []
    for num1 in range(len(filterLi)):

        for num2 in range(len(filterLi)):
            score = adjusted_rand_score(logProtObj[num1].clustResultObj.sampleClustSer.tolist()
                                        , logProtObj[num2].clustResultObj.sampleClustSer.tolist())
            ARITable[num1][num2] = score
            histogramLi.append(score)
            MaskTable[num1][num2] = False
            if score < smallScore:
                smallScore = score
                number1 = num1 + 1
                number2 = num2 + 1

    # show gene numbers barchart, distribution histogram and overlap heatmap

    sns.barplot(x="MAD", y='geneNum', data=geneNumDf)
    plt.show()
    sns.distplot(histogramLi)
    plt.show()
    sns.heatmap(ARITable, cmap="Reds", linewidths=.5, annot=True, mask=MaskTable, vmin=0, vmax=1, xticklabels=SD,
                yticklabels=SD)
    print(smallScore, number1, number2)
    plt.show()
    print(ARITable)


def silhouettePlotTest():
    rawDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")

    rawDataObj = DataAdapter.FilterByCutoff(rawDataObj, methodStr='MAD', cutOff=0.5)


    # protClustObj=GetSampleClust(protClustObj)
    #protClustObj.clustResultObj.sampleClustSer = pd.read_csv("kmeans1.csv", index_col=0, header=None)
    #protClustObj.clustResultObj.sampleClustSer = protClustObj.clustResultObj.sampleClustSer.iloc[:, 0]

    # protClustObj.clustParamObj.SetParam(method="hierarchicalClust", linkage="average",
    #                                   distance="euclidean", clustNum=5)
    # protClustObj.clustParamObj.SetParam(method="consensusClust", ccMethod="kmeansClust_nltk",
    #                                       clustNum=5, resample=0.8, minK=5, maxK=5, resample_times=1000,
    #                                       ccDistance='pcc')
    # protClustObj = ClusteringMethod.Clustering(rawDataObj, protClustObj)
    protClustObj2 = DataAdapter.LoadProtClustObj("hierarchical_cos_complete.pkl")
    protClustObj = DataAdapter.LoadProtClustObj("new.pkl")
    # protClustObj = DataAdapter.LoadProtClustObj("fig/dif_clust/consensus_nltk_euclidean.pkl")
    # print(protClustObj.clustResultObj.sampleClustSer.tolist())
    print(adjusted_rand_score(protClustObj2.clustResultObj.sampleClustSer.tolist()
                              , protClustObj.clustResultObj.sampleClustSer.tolist()))
    # protClustObj1 = DataAdapter.LoadProtClustObj("PassOverLap1_1.pkl")
    # protClustObj2 = DataAdapter.LoadProtClustObj("PassOverLap1_2.pkl")

    # compute silhouette_score
    Scoring.CalcEveryScore(rawDataObj, protClustObj2)

    # Scoring.CalcSampleSilhScore(rawDataObj, protClustObj)
    # Scoring.CalcSilhScore(rawDataObj, protClustObj)
    # Scoring.CalcClustSilhScore(rawDataObj, protClustObj)
    '''
    # print scoring results
    #print("partitionIndex: "+protClustObj.clustScoringObj.partitionIndex)
    #print("separationIndex: " + protClustObj.clustScoringObj.separationIndex )
    #print("silhScore: " + protClustObj.clustScoringObj.silhScore)
    print("purity: " + str(protClustObj.clustScoringObj.purity))
    print("separation index: "+str(protClustObj.clustScoringObj.separationIndex))
    print("giniScore: " + str(protClustObj.clustScoringObj.giniScore))
    print("silhScore: " + str(protClustObj.clustScoringObj.silhScore))
    print("clustSilhScoreSer: " + str(protClustObj.clustScoringObj.clustSilhScoreSer))
    print("sampleSilhScoreSer: " + str(protClustObj.clustScoringObj.sampleSilhScoreSer))
    print("featureAriDi: " + str(protClustObj.clustScoringObj.featureAriDi))
    print("featureAmiDi: " + str(protClustObj.clustScoringObj.featureAmiDi))
    print("featureEntropyDi: " + str(protClustObj.clustScoringObj.featureEntropyDi))
    print("featureMaxFscoreDi: " + str(protClustObj.clustScoringObj.featureMaxFscoreDi))
    print("featureVscoreDi: " + str(protClustObj.clustScoringObj.featureVscoreDi))
    print("featurePairedFscoreDi: " + str(protClustObj.clustScoringObj.featurePairedFscoreDi))
    '''

    # draw sihouette plot
    Visualization.DrawSilhouettePlot(rawDataObj, protClustObj2)


def DifferentClusteringResult(differentclusterfile="fig/dif_clust/DifferentClusterFile.csv"):
    """
    run different clustering methods listed in csv file and save them as .pkl files
    :param differentclusterfile:
    :return:
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    parameter = pd.read_csv('different clustering result parameter.csv', header=None, index_col=0)

    parameter = parameter.T
    parameter.reset_index(drop=True, inplace=True)
    print(parameter['linkage'])
    print(parameter)

    resultCount = len(parameter['method'])
    protClustObj = [ds_ProtClust() for x in range(resultCount)]
    for num in range(resultCount):
        ##protClustObj.clustParamObj.SetParam(method="hierarchicalClust", linkage="average",
        #                                    distance="euclidean", clustNum=5)
        protClustObj[num].clustParamObj.SetParam(method=parameter['method'][num], distance=parameter['distance'][num],
                                                 ccMethod=parameter['ccMethod'][num], linkage=parameter['linkage'][num],
                                                 clustNum=5, resample=float(parameter['resample'][num]),
                                                 resample_times=int(parameter['resample_times'][num])
                                                 , minK=5, maxK=5, ccDistance=parameter['ccDistance'][num]
                                                 , ccLinkage=parameter['ccLinkage'][num])

        # protClustObj.clustParamObj.SetParam(method="selfOrganizingMapClust", dimension=5,
        #                                   clustNum=5, learnRate=0.1, sigma=3, epochNum=800)
        protClustObj[num] = ClusteringMethod.Clustering(protDataObj, protClustObj[num])

        DataAdapter.DumpProtClustObj(protClustObj[num], _NameDifferentClusteringResult(parameter, num))
        gct = open(differentclusterfile, 'a')
        gct.writelines(str(protClustObj[num].clustResultObj.sampleClustSer.tolist())[1:-1] + '\n')

        gct.close()


def DifferentClusteringResult_giniARIScatterPlot():
    """
    use .pkl files to draw histogram, scatterplot and heatmap
    :return:
    """
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)

    parameter = pd.read_csv('different clustering result parameter.csv', header=None, index_col=0)

    parameter = parameter.T
    parameter.reset_index(drop=True, inplace=True)
    resultCount = len(parameter['method'])
    # DataAdapter.DumpProtClustObj(protClustObj, "hierarchicalProtClust.pkl")
    # DataAdapter.DumpRawDataObj(protDataObj, "RawData.pkl")

    ARITable = pd.DataFrame([[0.0 for x in range(resultCount)]] * resultCount)
    MaskTable = pd.DataFrame([[True for x in range(resultCount)]] * resultCount)
    logProtObj = [ds_ProtClust() for x in range(resultCount)]
    giniLi = [0.0 for x in range(resultCount)]

    for num1 in range(resultCount):
        logProtObj[num1] = DataAdapter.LoadProtClustObj(_NameDifferentClusteringResult(parameter, num1))
        Scoring.CalcGiniScore(protDataObj, logProtObj[num1])
        giniLi[num1] = logProtObj[num1].clustScoringObj.giniScore

    ariScatterLi = []
    giniScatterLi = []
    histogramLi = []

    for num1 in range(resultCount):

        for num2 in range(num1):
            score = adjusted_rand_score(logProtObj[num1].clustResultObj.sampleClustSer.tolist()
                                        , logProtObj[num2].clustResultObj.sampleClustSer.tolist())
            if score < 0.05:
                score = 0.00
            ariScatterLi.append(round(score, 2))
            giniScatterLi.append(_ChooseLargeGini(giniLi, num1, num2))
            ARITable[num1][num2] = round(score, 2)
            MaskTable[num1][num2] = False
            histogramLi.append(score)

    sns.distplot(histogramLi, bins=5, hist_kws=dict(edgecolor="black", linewidth=2))
    print(len(histogramLi))
    plt.xlim(0, 1.2)
    plt.show()
    sns.distplot(giniLi, bins=4, hist_kws=dict(edgecolor="black", linewidth=2))
    plt.xlim(0, 1.0)
    plt.show()
    # plt.show()
    print(ARITable)
    # fig, axes = plt.subplots(1, 1, dpi=300)
    # scatterDataDf = pd.DataFrame(list(zip(ariScatterLi,giniScatterLi)), columns=['ari', 'average gini'])
    # sns.scatterplot(x='ari', y='average gini', data=scatterDataDf, ax=axes)
    g = sns.heatmap(ARITable, cmap="Reds", linewidths=.5, annot=True, mask=MaskTable, vmax=1, fmt='.2f', xticklabels=1,
                    yticklabels=1)
    g.set(xticklabels=[x for x in range(1, resultCount + 1)], yticklabels=[x for x in range(1, resultCount + 1)])
    plt.show()
    # axes.set_xlim([0,1])
    # axes.set_ylim([0, 1])
    # axes.set_xticklabels([x for x in range(1, resultCount)])
    # axes.set_yticklabels([x for x in range(1,resultCount)])
    # fig.savefig('fig/dif_clust/difClust_scatergini.png')


def DifferentClusteringResult_separationGiniScatterPlot():
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)

    parameter = pd.read_csv('different clustering result parameter.csv', header=None, index_col=0)

    parameter = parameter.T
    parameter.reset_index(drop=True, inplace=True)
    resultCount = len(parameter['method'])
    # DataAdapter.DumpProtClustObj(protClustObj, "hierarchicalProtClust.pkl")
    # DataAdapter.DumpRawDataObj(protDataObj, "RawData.pkl")
    nameLi = []

    logProtObj = [ds_ProtClust() for x in range(resultCount)]
    silhoLi = [0.0 for x in range(resultCount)]
    separationLi = [0.0 for x in range(resultCount)]
    giniLi = [0.0 for x in range(resultCount)]

    for num1 in range(resultCount):
        logProtObj[num1] = DataAdapter.LoadProtClustObj(_NameDifferentClusteringResult(parameter, num1))
        Scoring.CalcSeparationIndex(protDataObj, logProtObj[num1])
        separationLi[num1] = round(logProtObj[num1].clustScoringObj.separationIndex, 3)
        Scoring.CalcSilhScore(protDataObj, logProtObj[num1])
        silhoLi[num1] = round(logProtObj[num1].clustScoringObj.silhScore, 3)
        Scoring.CalcGiniScore(protDataObj, logProtObj[num1])
        giniLi[num1] = round(logProtObj[num1].clustScoringObj.giniScore, 3)
        nameLi.append(_NameDifferentClusteringResult(parameter, num1).split("/", 2)[2].split(".", 1)[0])

    sns.distplot(separationLi, bins=5, hist_kws=dict(edgecolor="black", linewidth=2))
    plt.xlim(0, 1.0)
    plt.show()
    sns.distplot(silhoLi, bins=5, hist_kws=dict(edgecolor="black", linewidth=2))
    plt.show()
    '''
    shortNameDf = pd.read_csv('shorthand name list.csv', header=None)
    sihouDf = DataFrame({'name':shortNameDf[1],'Silhouette Score':silhoLi})
    #giniDf = DataFrame({'name': shortNameDf[1], 'Gini Score': giniLi})
    #giniDf = giniDf.sort_values(by=['Gini Score'])
    sihouDf = sihouDf.sort_values(by=['Silhouette Score'])
    fig, axes = plt.subplots(1, 1, dpi=300)
    #sns.barplot(y='name',x='Gini Score',data=giniDf,ax=axes)
    sns.barplot(y='name', x='Silhouette Score', data=sihouDf, ax=axes)
    axes.set_yticklabels(sihouDf['name'], fontdict={'fontsize':5.0})
    #fig.savefig('fig/dif_clust/difClust_giniBarplot.png')
    fig.savefig('fig/dif_clust/difClust_silhouetteBarplot.png')


    #print("test: "+str(stats.mannwhitneyu(histogramLi,[0.96,1],use_continuity=False, alternative='two-sided')))
    #sns.distplot(giniLi)
    #plt.show()
    #sns.distplot(silhoLi, bins=5, hist_kws=dict(edgecolor="black", linewidth=2))
    #plt.show()
    fig, axes = plt.subplots(1, 1, dpi=300)
    scatterDataDf = pd.DataFrame(list(zip(silhoLi,giniLi)), columns=['Silhouette Score', 'Gini Score'])
    sns.scatterplot(x='Silhouette Score', y='Gini Score', data=scatterDataDf, ax=axes)
    #axes.set_xlim([0,1])
    axes.set_ylim([0, 1])

    fig.savefig('fig/dif_clust/difClust_separationGiniScatter.png')
    print(giniLi)
    print(silhoLi)
    #pd.Series(nameLi).to_csv('n.csv')
    '''


def PCA_TSNETEST():
    """
    find
    :param resampleTimes: the number to run clustering, to find the 5 similar clustering we want
    :return:
    """
    rawDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")

    rawDataObj = DataAdapter.FilterByCutoff(rawDataObj, methodStr='MAD', cutOff=0.5)
    # clust1 = DataAdapter.LoadProtClustObj('new.pkl')
    # clust2 = DataAdapter.LoadProtClustObj('hierarchical_cos_complete.pkl')
    clust1 = ds_ProtClust()
    clust2 = ds_ProtClust()
    clust1.clustResultObj.sampleClustSer = pd.read_csv(
        '../../pathway_ontology_analysis/clustering_data/differentfilter_4/filter1.csv', header=None, index_col=0)
    clust2.clustResultObj.sampleClustSer = pd.read_csv(
        '../../pathway_ontology_analysis/clustering_data/differentfilter_4/filter2.csv', header=None, index_col=0)
    clust2.clustResultObj.sampleClustSer = clust2.clustResultObj.sampleClustSer.iloc[:, 0]
    clust1.clustResultObj.sampleClustSer = clust1.clustResultObj.sampleClustSer.iloc[:, 0]

    # protClustObj2 = DataAdapter.LoadProtClustObj("consensusProtClust.pkl")
    # protClustObj = DataAdapter.LoadProtClustObj("hierarchicalProtClust.pkl")
    # protClustObj = DataAdapter.LoadProtClustObj("hierarchical_cos_complete.pkl")
    # protClustObj = DataAdapter.LoadProtClustObj("PassOverLap1_1.pkl")
    # print(protClustObj.clustResultObj.sampleClustSer.tolist())
    print(adjusted_rand_score(clust1.clustResultObj.sampleClustSer.tolist()
                              , clust2.clustResultObj.sampleClustSer.tolist()))
    # protClustObj1 = DataAdapter.LoadProtClustObj("PassOverLap1_1.pkl")
    # protClustObj2 = DataAdapter.LoadProtClustObj("PassOverLap1_2.pkl")

    # compute silhouette_score

    # Scoring.CalcEveryScore(rawDataObj, protClustObj)
    Visualization.PlotPca(rawDataObj, clust1, dpi=300, path='fig/MADTOP400_PCA.png')
    Visualization.PlotPca(rawDataObj, clust2, dpi=300, path='fig/MADTOP1200_PCA.png',
                          colorLi=['#9467bd', '#d62728', '#2ca02c', '#1f77b4', '#ff7f0e'])
    '''
    for count in range(1, 100):
        Visualization.PlotTsne(rawDataObj, protClustObj, dpi=300, perplexity=8,
                               path='fig/goterm/0.26/gaussian/gaussianTSNE_'+str(count)+'.png')
    #ratioTable = Visualization.FindSampleOverlapHeatmap(protClustObj, protClustObj1)
    #DataAdapter.DumpProtClustObj(protClustObj1, 'hierarchical_cos_complete.pkl')

    ##   '#d62728' red, '#9467bd' purple, '#2ca02c' green,'#1f77b4' blue, '#ff7f0e'orange
    '''



def TestConHierarchicalInstability():
    # parameter = pd.read_csv('TestConHierarchicalInstability.csv', header=None, index_col=0)
    parameter = pd.read_csv('TestConHCpccInstability.csv', header=None, index_col=0)
    parameter = parameter.T
    parameter.reset_index(drop=True, inplace=True)
    print(parameter['linkage'])
    print(parameter)

    resultCount = len(parameter['resample'])

    for num in range(resultCount):
        TestConHierachical(float(parameter['resample'][num]), int(parameter['resample_times'][num]),
                           parameter['ccDistance'][num], parameter['ccLinkage'][num])


def TestConHierachical(resam, resample_times, ccDistance, ccLinkage, resample=30):
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")
    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)

    protClustObj = [ds_ProtClust() for x in range(resample)]
    ARITable = pd.DataFrame([[0.0 for x in range(resample)]] * resample)
    MaskTable = pd.DataFrame([[True for x in range(resample)]] * resample)
    # conprotObj = DataAdapter.LoadProtClustObj('fig/dif_clust/hierarchical_euclidean_complete.pkl')
    # conprotObj =[ds_ProtClust() for x in range(resample)]

    for i in range(resample):
        protClustObj[i].clustParamObj.SetParam(method="consensusClust", clustNum=5, ccMethod='hierarchicalClust',
                                               minK=5, maxK=5,
                                               resample=resam, resample_times=resample_times, ccDistance=ccDistance,
                                               ccLinkage=ccLinkage)
        protClustObj[i] = ClusteringMethod.Clustering(protDataObj, protClustObj[i])

        # conprotObj[i].clustParamObj.SetParam(method="hierarchicalClust", clustNum=5, minK=5,
        #                                     maxK=5, distance='pcc', linkage='average')
        # conprotObj[i] = ClusteringMethod.Clustering(protDataObj, conprotObj[i])
    # protClustObj.clustResultObj.sampleClustSer.to_csv("SOMresult.csv")
    # DataAdapter.DumpProtClustObj(protClustObj, "SOMProtClust.pkl")
    # print(adjusted_rand_score(
    #        conprotObj.clustResultObj.sampleClustSer.tolist(),
    #        protClustObj[i].clustResultObj.sampleClustSer.tolist()))

    # Scoring.CalcGiniScore(protDataObj, protClustObj)
    # print("gini: "+str(protClustObj.clustScoringObj.giniScore))

    for num1 in range(resample):
        for num2 in range(num1):
            score = adjusted_rand_score(
                protClustObj[num1].clustResultObj.sampleClustSer.tolist(),
                protClustObj[num2].clustResultObj.sampleClustSer.tolist())
            ARITable[num1][num2] = round(score, 2)
            MaskTable[num1][num2] = False

    fig, ax = plt.subplots(figsize=(18, 10))
    g = sns.heatmap(ARITable, cmap="Reds", linewidths=.5, annot=True, mask=MaskTable, vmin=0, fmt='.2f',
                    xticklabels=1, yticklabels=1, vmax=1, ax=ax)
    # plt.show()
    g.set(xticklabels=[x for x in range(1, resample + 1)], yticklabels=[x for x in range(1, resample + 1)])

    # fig.savefig('fig/conHierarchical/'+str(resam)+'_'+str(resample_times)+'_'+ccDistance+'_'+ccLinkage+'.png',dpi=300)
    fig.savefig(
        'fig/conHierarchical/pcc/' + str(resam) + '_' + str(
            resample_times) + '_' + ccDistance + '_' + ccLinkage + '.png',
        dpi=300)
    # fig.cla()


def ConHCpccCompare():
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")

    protDataObj = DataAdapter.FilterByCutoff(protDataObj, methodStr='MAD', cutOff=0.5)
    protClustObj = [ds_ProtClust() for x in range(3)]
    protClustObj[0] = DataAdapter.LoadProtClustObj("fig/dif_clust/consensus_hierarchical_pcc_average.pkl")
    protClustObj[1] = DataAdapter.LoadProtClustObj("fig/dif_clust/consensus_hierarchical_pcc_complete.pkl")
    protClustObj[2] = DataAdapter.LoadProtClustObj("fig/dif_clust/consensus_hierarchical_pcc_single.pkl")

    #####   compute and draw ARI heatmap
    ARITable = pd.DataFrame([[0.0 for x in range(len(protClustObj))]] * len(protClustObj))
    MaskTable = pd.DataFrame([[True for x in range(len(protClustObj))]] * len(protClustObj))
    giniLi = [0.0 for x in range(3)]
    for num1 in range(len(protClustObj)):

        for num2 in range(num1):
            score = adjusted_rand_score(protClustObj[num1].clustResultObj.sampleClustSer.tolist()
                                        , protClustObj[num2].clustResultObj.sampleClustSer.tolist())

            if score < 0.05:
                score = 0.00
            ARITable[num1][num2] = score
            MaskTable[num1][num2] = False
        Scoring.CalcGiniScore(protDataObj, protClustObj[num1])
        giniLi[num1] = round(protClustObj[num1].clustScoringObj.giniScore, 3)
    '''
    ### draw ari heatmap
    #g = sns.heatmap(ARITable, cmap="Reds", linewidths=.5, annot=True, mask=MaskTable, vmax=1, fmt='.2f', xticklabels=1,
    #                yticklabels=1)
    #g.set(xticklabels=['average','complete','single'], yticklabels=['average','complete','single'])
    #plt.show()

    ### draw pca
    for num1 in range(len(protClustObj)):
        Visualization.PlotPca(protDataObj, protClustObj[num1], dpi=300, path='fig/conHierarchical/pcc/'+str(num1+1)+'.png')


    giniDf = DataFrame({'Cluster Method': ['ConHC_pcc_aver','ConHC_pcc_comp','ConHC_pcc_single'], 'Gini Score': giniLi})
    giniDf = giniDf.sort_values(by=['Gini Score'])
    fig, axes = plt.subplots(1, 1, dpi=300)
    sns.barplot(x='Cluster Method', y='Gini Score', data=giniDf, ax=axes)
    axes.set_xticklabels(giniDf['Cluster Method'])

    fig.savefig('fig/conHierarchical/pcc/gini Barplot.png')
    '''
    ### draw sampleoverlap heatmap
    # for num1 in range(len(protClustObj)-1):
    Visualization.FindSampleOverlapHeatmap(protClustObj[0], protClustObj[1],
                                           path='fig/conHierarchical/pcc/sampleoverlap_ConHC_average_complete.png',
                                           dpi=300)

    #######


def _NameDifferentClusteringResult(parameter, num):
    if parameter['method'][num] == 'hierarchicalClust':
        return 'fig/dif_clust/hierarchical_' + parameter['distance'][num] + '_' + parameter['linkage'][num] + '.pkl'
    elif parameter['method'][num] == 'consensusClust':
        if parameter['ccMethod'][num] == 'hierarchicalClust':
            return 'fig/dif_clust/consensus_hierarchical_' + parameter['ccDistance'][num] + '_' + \
                   parameter['ccLinkage'][num] + '.pkl'
        else:
            return 'fig/dif_clust/consensus_nltk_' + parameter['ccDistance'][num] + '.pkl'


def GoTermVennTest(path1, path2, subtype1, subtype2, process="biological", figPath='fig/GoTermVenn/0.7/'):
    node1 = path1 + '/cluster' + str(subtype1) + "_up/" + process + "/cluster" + str(subtype1) + "up.txt"
    node2 = path2 + '/cluster' + str(subtype2) + "_up/" + process + "/cluster" + str(subtype2) + "up.txt"
    return CaseStudy.DrawGoTermOverlapVenn(node1, node2, figPath + str(subtype1) + '_' + str(subtype2) + '.png')





def GSEATest():
    protDataObj = DataAdapter.ReadFilesPath("Example/colonCancerProtRatio.csv", "Example/colonCancerMetadata.csv")

    rawDataObj = DataAdapter.FilterByCutoff(protDataObj, method='MAD', cutOff=0.5)

    Statistics.GenerateGctFile(protDataObj, path='GSEAGeneTable.gct')

    protClustObj = ds_ProtClust()

    protClustObj.clustParamObj.SetParam(filter="median", method="hierarchical", linkage="ward",
                                        distance="euclidean", clustNum=5)

    protClustObj = ClusteringMethod.clustering(rawDataObj, protClustObj)

    Statistics.GenerateClsFile(protClustObj)
    # protClustObj.clustResultObj.sampleClustSer.to_csv ("colon_consensus.csv")

    # temp = protClustObj.clustResultObj.sampleClustSer.tolist ()

def GoTermTest(choseGoNum=10):
    upOrDown = "up"
    process = "biological"
    clustNum = 5
    # aabb=Characterization.findGoTermOverlap()
    CaseStudy.DrawGoTermHeatMap(goTerm1ClustLi=[1, 2, 3, 4, 5], goTerm2ClustLi=[1, 2, 3, 4, 5], choseGoNum=choseGoNum,
                                goTerm1Str="consensus nltk euclidean",
                                goTerm2Str="filtermad0.5_colon_hierachical_0.05", upOrDownStr=upOrDown,
                                processStr=process, clusterNum=clustNum, dpi=300)


def NetworkTest(sub=5):
    # for subtype in range(1, sub + 1):
    subtype1 = 4
    # subtype2 = 4
    node1 = "D:\pathway_ontology_analysis/network analysis/networkanalyst/consensus nltk euclidean/" + "consensus2_node_table.csv"
    node2 = "D:\pathway_ontology_analysis/network analysis/networkanalyst/hierarchical_cosine/" + "hierarchical4_node_table.csv"
    CaseStudy.DrawNetworkOverlapGeneVenn(node1, node2, subtype1, 5)
    CaseStudy.DrawNetworkOverlapGeneVenn(node1, node2, subtype1, 2)




